﻿using UnityEngine;
using UnityEngine.UI;
using UnityEngine.EventSystems;
using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using LAViD.VLibras.Dictionary;
using LAViD.Structures;

// https://unity3d.com/pt/learn/tutorials/modules/beginner/live-training-archive/creating-scroll-lists-at-run-time

[System.Serializable]
public class ItemData {

	public string animationName;
	public Button.ButtonClickedEvent thingToDo;

}

public class ListManager : MonoBehaviour {
	
	private Trie signs;

	public GameObject sampleItemObject;
	public GameObject sampleLoadingItemObject;

	public List<string> itemList;
	private int index = 0;
	private const int OFFSET = 20;
	private int size = 0;

	public GameObject listBlock;
	//public GameObject bar;

	public Transform contentPanel;
	public ScrollRect scrollView;
	public InputField input;

	private bool isLoading = false;
	private GameObject loadingItem;

	private float itemHeight = 30F;

	void Start()
	{
		this.scrollView.onValueChanged.AddListener(checkScrollPosition);
		this.input.onValueChanged.AddListener(inputChanged);

		if (Screen.dpi < 140)
			this.itemHeight = 36;
		else if (Screen.dpi < 280)
			this.itemHeight = 50;
		else if (Screen.dpi < 400)
			this.itemHeight = 90;
		else if (Screen.dpi < 500)
			this.itemHeight = 120;
		else
			this.itemHeight = 90;

		this.gameObject.GetComponent<Dictionary>().AddOnLoadListener(load);
	}

	private void load()
	{
		this.signs = this.gameObject.GetComponent<Dictionary>().Signs;

		this.itemList = this.signs.StartsWith("");
		this.index = 0;
		this.size = itemList.Count;

		populateList();
	}

	public void checkScrollPosition(Vector2 scrollPosition)
	{
		if (scrollPosition.y <= 0.05F && ! this.isLoading)
			populateList();
	}

	public void inputChanged(string text)
	{
		this.itemList = this.signs.StartsWith(text.ToUpper());
		this.index = 0;
		this.size = itemList.Count;

		this.contentPanel.DetachChildren();
		foreach(GameObject go in GameObject.FindGameObjectsWithTag("clone"))
			Destroy(go);

		populateList();
		this.scrollView.verticalNormalizedPosition = 1F;
	}

	private void populateList()
	{
		int last = this.index + OFFSET;
		if (last > size) last = this.size;

		for (int i = index; i < last; i++)
		{
			string item = itemList[i];

			GameObject newItem = Instantiate (sampleItemObject) as GameObject;
			SampleItem sampleItem = newItem.GetComponent<SampleItem>();
			sampleItem.title.text = item;

			LayoutElement le = newItem.GetComponent<LayoutElement>();
			le.minHeight = this.itemHeight;

			sampleItem.GetComponent<Button>().onClick.AddListener(
				delegate {
					listBlock.SetActive(false);
				}
			);

			if (contentPanel.childCount == 0)
				newItem.transform.GetChild(1).gameObject.SetActive(false);

			newItem.transform.SetParent(contentPanel);
		}

		this.index = last;
	}

}
