﻿/**********************
********LAVID**********
***VLibras Project*****
*------------------------------------------------------------------------
*Description:
*
*This class needs an android plugin to call Dialogs Mobile APIs, working on both
* And IOS platforms.
* Export Video is responsible for making available the share video functionality.
*It works managing the button layers related to the funcionality and also making
* requests to VLibras server.
*---------------------------------------------------------------------------
*Plugin directory: 
* - Assets/Plugins/Android
*References:
* - http://docs.unity3d.com/ScriptReference/AndroidJavaClass.html
*------------------------------------------------------------------------
*Author: Thiago Filipe
*thiago.filipe@lavid.ufpb.br
***********************/


using UnityEngine;
using System;
using System.Collections;
using LAViD.Unity.Utils;


public class ExportVideo : MonoBehaviour {


    public ScreenManager screenManager;
    public MediaShareAndroid MediaShareAndroid;

    private const string SERVER_URL = "http://traducao.vlibras.gov.br/video";
    


    private static string gloss = "Gloss";


    /* Strings related to the Android Dialog*/
    private string title = "Download";
    private string message = "Para compartilhar é preciso realizar o download do vídeo. Tamanho do Download: ";
    private string yesOp = "DOWNLOAD";
    private string noOp = "CANCELAR";

    /* Data related to the Video*/

    private int videoSize = 0;
    private int bytes = 0;
    private string videoId = "";
    private string videoFile = "";
    private byte[] videoContent = null;

    /*Coroutine flags */

    private bool videoRequestRunning = false;
    private bool videoDownloadRunning = false;
    private bool videoStatusRunning = false;




    /*Getting called by PlayerManager, setting the gloss property*/

    public static void SetGloss(string _gloss)
    {
        PlayerLogger.Log("ExportVideo", "SetGloss", "Setting current gloss to gloss property from ExportVideo");
        gloss = _gloss;
        
    }   


    public void onClickExport()
    {
        PlayerLogger.Log("ExportVideo", "OnClickExport", "Starting Export...");

         NoodlePermissionGranter.PermissionRequestCallback = Permitted;
         NoodlePermissionGranter.GrantPermission(NoodlePermissionGranter.NoodleAndroidPermission.WRITE_EXTERNAL_STORAGE);
        


    }


    /* RunTime Android Permission being asked*/
    private void Permitted(bool PermissionGranted)
    {
        if (PermissionGranted)
        {
            int fiveTimesCount;

            if (PlayerPrefs.HasKey("fiveTimesCount"))
            {
                fiveTimesCount = PlayerPrefs.GetInt("fiveTimesCount");
                if(fiveTimesCount < 5)
                {
                    fiveTimesCount++;
                    PlayerPrefs.SetInt("fiveTimesCount", fiveTimesCount);
                    PlayerPrefs.Save();

                }
                else
                {
                    fiveTimesCount = 0;
                    PlayerPrefs.SetString("exportDialog", "declined");
                    PlayerPrefs.Save();
                }

            }else
            {
                PlayerPrefs.SetFloat("fiveTimesCount", 0);
                PlayerPrefs.Save();
            }
            bool accepted = false;

            string message = "A geração de vídeo pode levar algum tempo.";
            if (PlayerPrefs.HasKey("exportDialog"))
            {
                accepted = PlayerPrefs.GetString("exportDialog").Equals("accepted");
            }

            if (!accepted)
            {
                MobileNativeDialog nativeDialog;
                nativeDialog = new MobileNativeDialog("Exportar", message, "Continuar", "Cancelar");
                nativeDialog.OnComplete = OnExportDialog;

            }
            else
            {
                startExport();
            }

        }
        else
        {
            screenManager.showPermissionDialog("Para salvar e compartilhar as animações, permita que o Vlibras acesse mídia e arquivos de seu aparelho.", "Exportar Vìdeo");
           // new AndroidToast().showToast("Você precisa dá permissão ao aplicativo.", "LONG");
        }
        

    }

    private void OnExportDialog(MNDialogResult result)
    {
        if (result == MNDialogResult.YES)
        {
            PlayerPrefs.SetString("exportDialog", "accepted");
            PlayerPrefs.Save();
            startExport();

        }
        else if (result == MNDialogResult.NO)
        {
            PlayerLogger.Log("ExportVideo", "OnExportDialog", "Resetting states to default...");
            screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);

        }


    }

    /* Function that hides the export button and 
    * shows the progress bar while getting video info*/
    public void startExport()
    {
        new AndroidToast().showToast("Gerando Video...", "SHORT");
        screenManager.changeExportStates(ExportLayers.ExportLayer.Export_Layer, false);

        screenManager.changeExportStates(ExportLayers.ExportLayer.Progress_Layer, true);
        StartCoroutine("requestVideoInfo");
        PlayerLogger.Log("ExportVideo", "OnClickExport", "Making request to server.");

    }



    /* Function that gets called by the web request
     * hiding the progress bar and showing the download
     * button*/
    public void OnFinishGetVideoInfo()
    {
        PlayerLogger.Log("ExportVideo", "OnFinishGetVideoInfo", "Video is ready to be downloaded.");
        screenManager.changeExportStates(ExportLayers.ExportLayer.Progress_Layer, false);
        screenManager.changeExportStates(ExportLayers.ExportLayer.Download_Layer, true);
        PlayerLogger.Log("ExportVideo", "OnFinishGetVideoInfo", "Hiding progress loading and showing download button");

    }

    /* Function that asks the user whether downlaod or not
     * the video by presenting a Dialog Box.**/

    public void onDownloadClick()
    {
        MobileNativeDialog nativeDialog;

        nativeDialog = new MobileNativeDialog(title, message + ((videoSize / 1024f) / 1024f).ToString("0.00") + " MB.", yesOp, noOp);
        nativeDialog.OnComplete = OnComplete;

    }

    /** Delegate function which waits for YES or NO from the MobileDialog
     * and attach a action to each option**/

    public void OnComplete(MNDialogResult result)
    {
        if (result == MNDialogResult.YES)
        {
            new AndroidToast().showToast("Download iniciado.", "SHORT");
            PlayerLogger.Log("ExportVideo", "OnDownloadClick", "Starting Video Download...");
            StartCoroutine("videoDownload");
            screenManager.changeExportStates(ExportLayers.ExportLayer.Download_Layer, false);
            screenManager.changeExportStates(ExportLayers.ExportLayer.Progress_Download_Layer, true);
        } else if (result == MNDialogResult.NO)
        {
            PlayerLogger.Log("ExportVideo", "OnDownloadClick", "Resetting states to default...");
            screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);

        }
    }

    /* Function that shares the video downloaded.**/

    public void OnDownloadFinished()
    {
        PlayerLogger.Log("ExportVideo", "OnDownloadFinished", "Video Downloaded, able to share it.");
        screenManager.changeExportStates(ExportLayers.ExportLayer.Progress_Download_Layer, false);
        screenManager.changeExportStates(ExportLayers.ExportLayer.Share_Layer, true);
    }

    /*Cancels the VideoRequest or VideoDownload by interrupting the coroutine and resetting values*/

    public void OnCancel()
    {
        if (videoRequestRunning || videoStatusRunning)
        {
            StopCoroutine("requestVideoInfo");
            StopCoroutine("VideoStatus");

            PlayerLogger.Log("ExportVideo", "OnRequestCancel", "User canceled the request, resetting states to default.");

        } else if (videoDownloadRunning)
        {
            StopCoroutine("videoDownload");
            PlayerLogger.Log("ExportVideo", "videoDownload", "User canceled the download, resetting states to default.");
        }
       

        videoId = "";
        videoSize = 0;
        videoContent = null;
        screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
    }

    /*With sharing layer active, gets called when the user clicks on it and makes
     * the video available to share*/

    public void OnShareVideo()
    {
        if (videoContent != null)
        {
            Action errorAction = () =>
            {
                screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                new AndroidToast().showToast("Erro ao compartilhar mídia.", "LONG");
            };

            StartCoroutine(MediaShareAndroid.ShareIntent(errorAction));
        }

        else
        {
            new AndroidToast().showToast("Erro ao compartilhar mídia.", "LONG");
            screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
        }


    }

    /*Coroutine that makes the video information request*/

    private IEnumerator requestVideoInfo()
    {
        
            videoRequestRunning = true;
            WWWForm JsonRequest = new WWWForm();
            JsonRequest.AddField("gloss", gloss);
            WWW videoInfoRequest = new WWW(SERVER_URL, JsonRequest);

        PlayerLogger.Log("ExportVideo", "requestVideoInfo", "Request for: " + SERVER_URL);

        yield return WaitForResponse(videoInfoRequest);

         try
            {
                if (!videoInfoRequest.isDone)
                {

                    screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.CONNECTION_TIMEOUT_FAILURE);
                    PlayerLogger.Log("ExportVideo", "requestVideoInfo", "TimeOut");
                  
                }
                else if (videoInfoRequest.error != null)
                {

                    screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                if (videoInfoRequest.error.ToLower().Contains("connection refused"))
                {
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.ERROR_CONNECTION_FAILURE);

                }
                else
                {
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.INTERNET_CONNECTION_FAILURE);
                }
                    PlayerLogger.Log("ExportVideo", "requestVideoInfo", "(WWW) Error: " + videoInfoRequest.error);
                    
                }
                else if (videoInfoRequest.responseHeaders.Count == 0)
                {
                    screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                    PlayerLogger.Log("ExportVideo", "requestVideoInfo", "(WWW) Unsucessful Answer");
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.ERROR_CONNECTION_FAILURE);
                    


                }
                else if (!videoInfoRequest.responseHeaders["STATUS"].Contains("200"))
                {
                    screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                    PlayerLogger.Log("ExportVideo", "requestVideoInfo", "Unsuccessful answer (" + videoInfoRequest.responseHeaders["STATUS"] + ").");
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.ERROR_CONNECTION_FAILURE);
                   

                }
                else if (String.IsNullOrEmpty(videoInfoRequest.text))
                {

                    screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.DEFAULT);
                    PlayerLogger.Log("ExportVideo", "requestVideoInfo", "Empty answer.");
                    
                }
                else
                {
                    PlayerLogger.Log("ExportVideo", "requestVideoInfo", "Answer: " + videoInfoRequest.text);

                    FileContent fileContent = new FileContent();
                    fileContent = FileContent.CreateFromJSON(videoInfoRequest.text);
                    videoId = fileContent.id;                
                    StartCoroutine("VideoStatus");

                }


            }
            finally
            {
                videoRequestRunning = false;



            }
        

        yield break;
       
    }

    private IEnumerator VideoStatus()
    {
        videoStatusRunning = true;
        
        while (true)
        {
            WWW videoStatusRequest = new WWW(SERVER_URL + "/status/" + videoId);
            Debug.Log(SERVER_URL + "/" + WWW.EscapeURL("status") + "/" + WWW.EscapeURL(videoId));


            yield return WaitForResponse(videoStatusRequest);

            try
            {
                if (!videoStatusRequest.isDone)
                {

                    screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.CONNECTION_TIMEOUT_FAILURE);
                    PlayerLogger.Log("ExportVideo", "VideoStatus", "TimeOut");
                    break;

                }
                else if (videoStatusRequest.error != null)
                {

                    screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                    if (videoStatusRequest.error.ToLower().Contains("connection refused"))
                    {
                        GetConnectionStatusError(ExportLayers.ConnectionStatusError.ERROR_CONNECTION_FAILURE);

                    }
                    else
                    {
                        GetConnectionStatusError(ExportLayers.ConnectionStatusError.INTERNET_CONNECTION_FAILURE);
                    }
                    PlayerLogger.Log("ExportVideo", "VideoStatus", "(WWW) Error: " + videoStatusRequest.error);
                    break;

                }
                else if (videoStatusRequest.responseHeaders.Count == 0)
                {
                    screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                    PlayerLogger.Log("ExportVideo", "VideoStatus", "(WWW) Unsucessful Answer");
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.ERROR_CONNECTION_FAILURE);
                    break;



                }
                else if (!videoStatusRequest.responseHeaders["STATUS"].Contains("200"))
                {
                    screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                    PlayerLogger.Log("ExportVideo", "VideoStatus", "Unsuccessful answer (" + videoStatusRequest.responseHeaders["STATUS"] + ").");
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.ERROR_CONNECTION_FAILURE);
                    break;

                }
                else if (String.IsNullOrEmpty(videoStatusRequest.text))
                {

                    screenManager.changeExportStates(ExportLayers.ExportLayer.All, true);
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.DEFAULT);
                    PlayerLogger.Log("ExportVideo", "VideoStatus", "Empty answer.");
                    break;

                }
                else
                {
                    PlayerLogger.Log("ExportVideo", "VideoStatus", "Answer: " + videoStatusRequest.text);

                    
                    FileContent fileContent = new FileContent();
                    fileContent = FileContent.CreateFromJSON(videoStatusRequest.text);                    
                    if(fileContent.status.Equals("success"))
                    {
                        
                        videoFile = fileContent.file;
                        videoSize = int.Parse(fileContent.size);
                        OnFinishGetVideoInfo();
                        break;
                    }

                    yield return new WaitForSeconds(1f);        


                }


            }
            finally
            {
                videoRequestRunning = false;
            }

            


        }// while
        videoStatusRunning = false;
        yield break;
    }



    /*Coroutine that downloads the video and update the progress download layer*/

    private IEnumerator videoDownload()
    {
        videoDownloadRunning = true;
        WWW DownloadVideo = new WWW(SERVER_URL + "/" + videoFile);

        while (!DownloadVideo.isDone)
        {
            screenManager.updateProgressDownloadSprite(DownloadVideo.progress);

            yield return null;
        }

        try
        {

            if (DownloadVideo.error != null)
            {
                if (DownloadVideo.error.ToLower().Contains("connection refused"))
                {
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.ERROR_CONNECTION_FAILURE);

                }
                else
                {
                    GetConnectionStatusError(ExportLayers.ConnectionStatusError.INTERNET_CONNECTION_FAILURE);
                }
                ErrorDownloading();

            }
            else if (DownloadVideo.responseHeaders.Count == 0)
            {
                GetConnectionStatusError(ExportLayers.ConnectionStatusError.ERROR_CONNECTION_FAILURE);
                ErrorDownloading();

            }
            else if (!DownloadVideo.responseHeaders["STATUS"].Contains("200"))
            {
                GetConnectionStatusError(ExportLayers.ConnectionStatusError.ERROR_CONNECTION_FAILURE);
                ErrorDownloading();

            }
            else if (DownloadVideo.bytesDownloaded == videoSize)
            {
                bytes = DownloadVideo.bytesDownloaded;
                videoContent = DownloadVideo.bytes;
                Action callBack = () => { OnDownloadFinished(); new AndroidToast().showToast("Vídeo salvo na Galeria.", "SHORT"); };
                Action errorWriting = () => { screenManager.changeExportStates(ExportLayers.ExportLayer.All, true); };
                videoDownloadRunning = false;
                StartCoroutine(MediaShareAndroid.WriteOnAndroid(videoContent, bytes, System.DateTime.Now.ToString("dd_MM_yyyy_HH_mm_ss"), callBack, errorWriting));

            }
            else
            {
                GetConnectionStatusError(ExportLayers.ConnectionStatusError.DOWNLOAD_CONNECTION_FAILURE);
                ErrorDownloading();
            }


        }
        finally
        {
            videoDownloadRunning = false;
        }
    }

    /* In case of error during the video download, the user gets a new chance to download the file, but
     * is also able to start a new fresh animation and export it */
    private void ErrorDownloading()
    {
        screenManager.changeExportStates(ExportLayers.ExportLayer.Progress_Download_Layer, false);
        screenManager.changeExportStates(ExportLayers.ExportLayer.Download_Layer, true);
        screenManager.changeExportStates(ExportLayers.ExportLayer.OnLockExport, false);
        screenManager.changeExportStates(ExportLayers.ExportLayer.OnLockShare, false);
    }

    public IEnumerator WaitForResponse(WWW www, Events.RequestSuccess success, Events.RequestError error)
    {
        yield return Methods.WaitForResponse(www, 15f, success, error);

    }

    protected IEnumerator WaitForResponse(WWW www)
    {
        yield return WaitForResponse(www, null, null);
    }

    /* Error states to show a Toast to user*/
    private void GetConnectionStatusError(ExportLayers.ConnectionStatusError error)
    {
        switch (error)
        {
            case ExportLayers.ConnectionStatusError.CONNECTION_TIMEOUT_FAILURE:
                new AndroidToast().showToast("Erro de conexão. Tente novamente.", "LONG");
                break;
            case ExportLayers.ConnectionStatusError.DOWNLOAD_CONNECTION_FAILURE:
                new AndroidToast().showToast("Erro ao efetuar download, tente novamente.", "LONG");
                break;
            case ExportLayers.ConnectionStatusError.INTERNET_CONNECTION_FAILURE:
                new AndroidToast().showToast("Não há acesso à internet. Verifique sua conexão", "LONG");
                break;
            case ExportLayers.ConnectionStatusError.ERROR_CONNECTION_FAILURE:
                new AndroidToast().showToast("Problema na conexão. Aguarde e tente novamente.", "LONG");
                break;
            default:
                new AndroidToast().showToast("Ocorreu um erro. Estamos trabalhando para solucioná-lo!", "LONG");
                break;
        }

    }





}
