﻿/**********************
********LAVID**********
***VLibras Project*****
*------------------------------------------------------------------------
*Description:
*
*This class is a Android Plugin that makes available the sharing function.
*---------------------------------------------------------------------------
*Plugin directory: 
* - Assets/Plugins/Android
*References:
* - http://docs.unity3d.com/ScriptReference/AndroidJavaClass.html
*------------------------------------------------------------------------
*Author: Thiago Filipe
*thiago.filipe@lavid.ufpb.br
***********************/
using UnityEngine;
using System.Collections;
using System.IO;
using System;
using LAViD.Unity.Utils;



public class MediaShareAndroid : MonoBehaviour
{

    private string MediaType = "video/*";
    
    private string SubtitleMessage = "VLibras na Playstore - https://play.google.com/store/apps/details?id=com.lavid.vlibrasdroid&hl=pt_BR";

    private string AndroidPath;
    
    private static string destination = "";
    private string Vlibras = "DCIM/VLibras";
    private string Filen = "VLibrasVideo_";
    private static string FilePath = "";

    private byte[] videoContent;
    private int bytes;

    private void Awake()
    {
        AndroidPath = AndroidJavaUti.CurrentSDCardPath;
    }


    /*Makes sure that Vlibras Folder gets created and write the video bytes in it.*/

    public IEnumerator WriteOnAndroid(byte[] videoContent, int bytes,string CurrentTime, Action finishWriting, Action errorWriting)
    {
        if (Application.platform == RuntimePlatform.Android)
        {

            FilePath = Filen + CurrentTime + ".mp4";
            destination = Path.Combine(AndroidPath, Vlibras);


            /* Checking if folder Vlibras exists*/

            DirectoryInfo dir = new DirectoryInfo(destination);
            if (!dir.Exists)
            {
                dir.Create();
            }

            try
            {
                FileStream SourceStream = new FileStream(Path.Combine(destination, FilePath), FileMode.OpenOrCreate);
                
                if (SourceStream.CanWrite)
                {
                    SourceStream.Write(videoContent, 0, bytes);
                    SourceStream.Close();

                    Refresh(FilePath,finishWriting);
                    
                    yield break;

                }
                else
                {
                    new AndroidToast().showToast("Erro ao salvar vídeo.", "LONG");
                    errorWriting();
                }


            }
            catch (Exception e)
            {
                PlayerLogger.Log("MediaShareAndroid", "WriteOnAndroid", "Message: " + e.Message);
                PlayerLogger.Log("MediaShareAndroid", "WriteOnAndroid", "Message: " + e.StackTrace);               
                new AndroidToast().showToast("Você precisa dá permissão ao aplicativo", "LONG");
               
                errorWriting();
            }



        }




    }

    /* Calling Android Services to refresh the video path, making available to show in gallery*/

    private void Refresh(string FilePath, Action finishWriting)
    {
        if (Application.platform == RuntimePlatform.Android)
        {
            /*
                        AndroidJavaClass jcUnityPlayer = new AndroidJavaClass("com.unity3d.player.UnityPlayer");
                        AndroidJavaObject joActivity = jcUnityPlayer.GetStatic<AndroidJavaObject>("currentActivity");
                        AndroidJavaObject joContext = joActivity.Call<AndroidJavaObject>("getApplicationContext");
                        AndroidJavaClass jcMediaScannerConnection = new AndroidJavaClass("android.media.MediaScannerConnection");
                        //AndroidJavaClass jcEnvironment = new AndroidJavaClass("android.os.Environment");
                        // AndroidJavaObject joExDir = jcEnvironment.CallStatic<AndroidJavaObject>("getExternalStorageDirectory");
                        string path = Path.Combine(destination,FilePath);
                        jcMediaScannerConnection.CallStatic("scanFile", joContext, new string[] { path }, new string[] { "video/mp4" }, null);
                        */
            Debug.Log("**create activity instance");
            AndroidJavaClass classPlayer = new AndroidJavaClass("com.unity3d.player.UnityPlayer");
            AndroidJavaObject objActivity = classPlayer.GetStatic<AndroidJavaObject>("currentActivity");

            Debug.Log("**create Uri class");
            AndroidJavaClass classUri = new AndroidJavaClass("android.net.Uri");

            Debug.Log("**create Intent object");
            AndroidJavaObject objIntent = new AndroidJavaObject("android.content.Intent", new object[2] { "android.intent.action.MEDIA_SCANNER_SCAN_FILE", classUri.CallStatic<AndroidJavaObject>("parse", "file://" + Path.Combine(destination,FilePath)) });

            Debug.Log("**call sendBroadcast");
            objActivity.Call("sendBroadcast", objIntent);
            finishWriting(); 


        }
    }


    /* Calling Android Services to create Share Intent*/

    public IEnumerator ShareIntent(Action errorAction) {

        if (Application.platform == RuntimePlatform.Android)
        {
            if (File.Exists(Path.Combine(destination, FilePath)))
            {

                // block to open the file and share it ------------START
                AndroidJavaClass intentClass = new AndroidJavaClass("android.content.Intent");
                AndroidJavaObject intentObject = new AndroidJavaObject("android.content.Intent");
                intentObject.Call<AndroidJavaObject>("setAction", intentClass.GetStatic<string>("ACTION_SEND"));

                AndroidJavaClass uriClass = new AndroidJavaClass("android.net.Uri");
                AndroidJavaObject uriObject = uriClass.CallStatic<AndroidJavaObject>("parse", "file://" + Path.Combine(destination, FilePath));

                intentObject.Call<AndroidJavaObject>("putExtra", intentClass.GetStatic<string>("EXTRA_STREAM"), uriObject);
                intentObject.Call<AndroidJavaObject>("setType", MediaType);

                //add data to be passed to the other activity i.e., the data to be sent
                intentObject.Call<AndroidJavaObject>("putExtra", intentClass.GetStatic<string>("EXTRA_SUBJECT"), "VLibras");
                intentObject.Call<AndroidJavaObject>("putExtra", intentClass.GetStatic<string>("EXTRA_TEXT"), SubtitleMessage);
                intentObject.Call<AndroidJavaObject>("setType", "video/*");
                AndroidJavaClass unity = new AndroidJavaClass("com.unity3d.player.UnityPlayer");
                AndroidJavaObject jChooser = intentClass.CallStatic<AndroidJavaObject>("createChooser", intentObject, "Compartilhar via");
                AndroidJavaObject currentActivity = unity.GetStatic<AndroidJavaObject>("currentActivity");
                currentActivity.Call("startActivity", jChooser);
                yield break;
            }
            else
            {
                errorAction();

                yield break;
            }
        }
    }

}


